
/******************************************************************************
* MODULE     : tree.gen.cc
* DESCRIPTION: fixed size trees with reference counting
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <tree.gen.h>
#include <array.gen.cc>

#module code_tree
#import tree
#import code_array (tree)

tree::tree (string l, tree t1):
  rep (new tree_rep(l, array<tree>(1))) {
  rep->a[0]=t1;
}

tree::tree (string l, tree t1, tree t2):
  rep (new tree_rep(l, array<tree>(2))) {
  rep->a[0]=t1;
  rep->a[1]=t2;
}

tree::tree (string l, tree t1, tree t2, tree t3):
  rep (new tree_rep(l, array<tree>(3))) {
  rep->a[0]=t1;
  rep->a[1]=t2;
  rep->a[2]=t3;
}

tree::tree (string l, tree t1, tree t2, tree t3, tree t4):
  rep (new tree_rep(l, array<tree>(4))) {
  rep->a[0]=t1;
  rep->a[1]=t2;
  rep->a[2]=t3;
  rep->a[3]=t4;
}

void
tree::resize (int n2) {
  rep->a.resize(n2);
}

tree
tree::operator () (int begin, int end) {
  int i;
  tree r (rep->label, end-begin);
  for (i=begin; i<end; i++) r[i-begin]= rep->a[i];
  return r;
}

bool
operator == (tree t, tree u) {
  return (t->label==u->label) && (t->a==u->a);
}

bool
operator != (tree t, tree u) {
  return (t->label!=u->label) || (t->a!=u->a);
}

tree
copy (tree t) {
  int i, n=N(t);
  tree t2 (copy (t->label), n);
  for (i=0; i<n; i++) t2[i]= copy (t[i]);
  return t2;
}

tree&
operator << (tree& t, tree t2) {
  t->a << t2;
  return t;
}

tree&
operator << (tree& t, array<tree> a) {
  t->a << a;
  return t;
}

ostream&
operator << (ostream& out, tree t) {
  int i;
  
  out << t->label;
  if (N(t)==0) return out;
  out << " (";
  for (i=0; i< N(t)-1; i++)
    out << t[i] << ", ";
  out << t[i] << ")";
  return out;
}

#endmodule // code_tree
