
/******************************************************************************
* MODULE     : tree.gen.h
* DESCRIPTION: fixed size trees with reference counting
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <basic.gen.h>
#include <string.gen.h>
#include <array.gen.h>

#module tree
#import basic
#import string

/******************************************************************************
* The tree class 'tree'
******************************************************************************/

class tree_rep;
class tree {
#import concrete (tree, tree_rep)
  inline tree ();
  inline tree (string l, int n=0);
  inline tree (char* l);
  tree (string l, tree t1);
  tree (string l, tree t1, tree t2);
  tree (string l, tree t1, tree t2, tree t3);
  tree (string l, tree t1, tree t2, tree t3, tree t4);
  inline tree& operator [] (int i);
  void resize (int n2);
  tree operator () (int start, int end);
};

#import array (tree)
class tree_rep: concrete_struct {
public:
  string label;
  array<tree> a;
  inline tree_rep (string l, array<tree> a2);
  friend class tree;
};

/******************************************************************************
* Routines for trees
******************************************************************************/

#import code_concrete (tree, tree_rep)
inline tree::tree ():
  rep (new tree_rep(string (), array<tree>())) {}
inline tree_rep::tree_rep (string l, array<tree> a2):
  label(l), a(a2) {}
inline tree::tree (string l, int n):
  rep (new tree_rep(l, array<tree>(n))) {}
inline tree::tree (char* l):
  rep (new tree_rep(string(l), array<tree>())) {}
inline tree& tree::operator [] (int i) { return rep->a[i]; }
extern inline int N (tree t) { return N(t->a); }

tree copy (tree t);
bool operator == (tree t, tree u);
bool operator != (tree t, tree u);
tree& operator << (tree& t, tree t2);
tree& operator << (tree& t, array<tree> a);
ostream& operator << (ostream& out, tree t);

inline bool is_func (tree t, string s) {
  return (N(t)>0) && (t->label==s); }
inline bool is_func (tree t, string s, int i) {
  return (N(t)==i) && (t->label==s); }
inline bool is_func (tree t, char* s) {
  return (N(t)>0) && (t->label==s); }
inline bool is_func (tree t, char* s, int i) {
  return (N(t)==i) && (t->label==s); }

#endmodule // tree
