
/******************************************************************************
* MODULE     : evaluate.gen.cc
* DESCRIPTION: Execution of scheme commands via guile
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <scheme.gen.h>

#module code_evaluate
#import scheme
#include <gh.h>

extern void initialize_glue ();
extern scheme_tree scm_to_scheme_tree (SCM p);

/******************************************************************************
* Installation of guile and initialization of guile
******************************************************************************/

void
install_guile (int argc, char** argv, void (*call_back) (int, char**)) {
  gh_enter (argc, argv, (void (*)(...)) ((void*) call_back));
}

void
initialize_guile () {
  char* init_prg =
    "(define TeXmacs-guile-error-handler
       (lambda args
         (object->string args)))

     (define object->string
       (lambda (obj)
         (call-with-output-string
           (lambda (port) (write obj port)))))

     (define TeXmacs-guile-eval
       (lambda (s)
         (catch
           #t
           (lambda ()
             (object->string (eval-string s)))
           TeXmacs-guile-error-handler)))

     (define TeXmacs-guile-eval-variant
       (lambda (s)
         (catch
           #t
           (lambda () (eval-string s))
           TeXmacs-guile-error-handler)))

     (define buffer-menu '())
     (define project-buffer-menu '())";

  gh_eval_str (init_prg);
  initialize_glue ();
}

/******************************************************************************
* Evaluation of scheme commands
******************************************************************************/

static string
back_slash (string s) {
  int i;
  string r;
  for (i=0; i<N(s); i++)
    switch (s[i]) {
    case '\\': r << "\\\\"; break;
    case '\"': r << "\\\""; break;
    default: r << s[i];
    }
  return r;
}

/*
static string
start (string s) {
  if (N(s)<200) return s;
  else return s(0,200) * "...";
}
*/

bool
eval_scheme_any (string s, string& r) {
  // cout << "Eval any] " << start (s) << "\n";
  string prg= "(TeXmacs-guile-eval \"" * back_slash (s) * "\")";
  char* _s= as_charp (prg);
  SCM result= gh_eval_str (_s);
  delete[] _s;

  int len_r;
  char* _r= gh_scm2newstr (result, &len_r);
  r= _r;
  free (_r);

  if (r == "#<unspecified>") r= "";
  // cout << "Yields] " << start (r) << "\n";
  return FALSE;
}

bool
eval_scheme_bool (string s, bool& r) {
  // cout << "Eval bool] " << start (s) << "\n";
  string prg= "(TeXmacs-guile-eval-variant \"" * back_slash (s) * "\")";
  char* _s= as_charp (prg);
  SCM result= gh_eval_str (_s);
  delete[] _s;
  if (gh_boolean_p (result)) {
    r= gh_scm2bool (result);
    // cout << "Yields] " << r << "\n";
    return FALSE;
  }
  else {
    r= FALSE;
    // cout << "Yields] Type mismatch\n";
    return TRUE;
  }
}

bool
eval_scheme_scheme_tree (string s, scheme_tree& r) {
  // cout << "Eval scheme tree] " << start (s) << "\n";
  string prg= "(TeXmacs-guile-eval-variant \"" * back_slash (s) * "\")";
  char* _s= as_charp (prg);
  SCM result= gh_eval_str (_s);
  delete[] _s;
  r= scm_to_scheme_tree (result);
  // cout << "Yields] " << r << "\n";
  return FALSE;
}

#endmodule // code_evaluate
